mod pagination;

use sea_orm::{sea_query::IntoCondition, *};

use axum::extract::{Path, Query, State};
use axum::routing::get;
use axum::Json;
use axum::Router;

use pagination::PaginatedResponse;

use crate::db::{self, *};

pub fn router() -> Router<crate::Global> {
    Router::new()
        .route("/repos", get(get_repos))
        .route("/repos/:id", get(get_single_repo))
        .route("/packages", get(get_packages))
        .route("/packages/:id", get(get_single_package))
}

async fn get_repos(
    State(global): State<crate::Global>,
    Query(pagination): Query<pagination::Query>,
    Query(filter): Query<db::query::repo::Filter>,
) -> crate::Result<Json<PaginatedResponse<db::repo::Model>>> {
    let page = pagination.page.unwrap_or(1) - 1;
    let per_page = pagination.per_page.unwrap_or(25);

    let paginator = Repo::find()
        .filter(filter)
        .order_by_asc(package::Column::Id)
        .paginate(&global.db, pagination.per_page.unwrap_or(25));
    let items = paginator
        .fetch_page(pagination.page.unwrap_or(1) - 1)
        .await?;
    let total_pages = paginator.num_pages().await?;

    Ok(Json(PaginatedResponse {
        page,
        per_page,
        total_pages,
        count: items.len(),
        items,
    }))
}

async fn get_single_repo(
    State(global): State<crate::Global>,
    Path(id): Path<i32>,
) -> crate::Result<Json<db::repo::Model>> {
    let repo = global
        .db
        .repo
        .by_id(id)
        .await?
        .ok_or(axum::http::StatusCode::NOT_FOUND)?;

    Ok(Json(repo))
}

async fn get_packages(
    State(global): State<crate::Global>,
    Query(pagination): Query<pagination::Query>,
    Query(filter): Query<db::query::package::Filter>,
) -> crate::Result<Json<PaginatedResponse<db::package::Model>>> {
    let (total_pages, pkgs) = global
        .db
        .pkg
        .page(
            pagination.per_page.unwrap_or(25),
            pagination.page.unwrap_or(1) - 1,
            filter,
        )
        .await?;

    Ok(Json(pagination.res(total_pages, pkgs)))
}

async fn get_single_package(
    State(global): State<crate::Global>,
    Path(id): Path<i32>,
) -> crate::Result<Json<crate::db::FullPackage>> {
    let entry = global
        .db
        .pkg
        .full(id)
        .await?
        .ok_or(axum::http::StatusCode::NOT_FOUND)?;

    Ok(Json(entry))
}
